@@ -1,11 +1,10 @@
 """For space rocks."""
-import os
-import sys
-
+from pathlib import Path
 import rich
 
 # rocks modules
-from . import datacloud, definitions, plots, ssodnet, utils
+# rocks.plots is lazy-loaded as it is expensive
+from . import datacloud, definitions, ssodnet, utils
 
 # Expose API to user
 from .core import Rock
@@ -14,18 +13,22 @@ from .resolve import identify
 
 # ------
 # Path definitions required throughout the code
-PATH_CACHE = os.path.join(os.path.expanduser("~"), ".cache/rocks")
-PATH_INDEX = os.path.join(PATH_CACHE, "index.pkl")
+PATH_CACHE = Path.home() / ".cache/rocks"
+PATH_INDEX = PATH_CACHE / "index"
 PATH_META = {
-    "description": os.path.join(PATH_CACHE, "ssoCard_description.json"),
-    "units": os.path.join(PATH_CACHE, "ssoCard_units.json"),
+    "description": PATH_CACHE / "ssoCard_description.json",
+    "units": PATH_CACHE / "ssoCard_units.json",
 }
 
+# Dict to hold the asteroid name-number indices at runtime
+INDEX = {}
+
 # ------
 # Welcome to rocks
-__version__ = "1.4.6"
+__version__ = "1.4.11"
 
 GREETING = fr"""
+                _
                | |
  _ __ ___   ___| | _____
 | '__/ _ \ / __| |/ / __|
@@ -43,9 +46,15 @@ Some metadata is required to be present in the cache directory.
 
 # ------
 # Check for existence of index file and cache directory
-os.makedirs(PATH_CACHE, exist_ok=True)
+if not PATH_INDEX.is_dir():
 
-if not os.path.isfile(PATH_INDEX):
     rich.print(GREETING)
+
+    # Just for a while
+    if (Path.home() / ".cache/rocks/index.pkl").is_file():
+        (Path.home() / ".cache/rocks/index.pkl").unlink()
+
+    PATH_INDEX.mkdir(parents=True)
     utils._build_index()
+
     rich.print("\nAll done. Find out more by running [green]$ rocks docs[/green]\n")
