@@ -2,7 +2,7 @@
 """Implement the Rock class and other core rocks functionality."""
 import datetime as dt
 import json
-from typing import List, Optional
+from typing import Dict, List, Optional
 import warnings
 
 import numpy as np
@@ -13,58 +13,6 @@ import rich
 import rocks
 
 
-# ------
-# Validators
-def ensure_list(value):
-    """Ensure that parameters are always a list.
-    Some parameters are a dict if it's a single reference and a list otherwise.
-
-    Further replaces all None values by empty dictionaries.
-    """
-    if isinstance(value, dict):
-        value = [value]
-
-    for i, v in enumerate(value):
-        if v is None:
-            value[i] = {}
-    return value
-
-
-def merge_entries(value):
-    """Turn list of dicts into dict of lists."""
-
-    if isinstance(value, list):  # taxonomy
-        pass
-    elif isinstance(value, dict):  # spin
-        value = list(value.values())
-
-    return_dict = {}
-
-    # Take keys of first dict
-    for key in value[0]:
-
-        # Turn all dicts in the list into lists
-        return_dict[key] = []
-
-        for v in value:
-            if key in v:
-                return_dict[key].append(v[key])
-            else:
-                # This entry of the current parameter does not have all
-                # values (eg "period" for Spin)
-
-                # Add the correct NaN value
-                if isinstance(value[0][key], dict):
-                    return_dict[key].append({})
-                elif isinstance(value[0][key], (float, int)):
-                    return_dict[key].append(np.nan)
-                elif isinstance(value[0][key], str):
-                    return_dict[key].append(None)
-    return return_dict
-
-    # return {key: [entry[key] for entry in value] for key in value[0]}
-
-
 # ------
 # ssoCard as pydantic model
 
@@ -116,6 +64,64 @@ class Bibref(Parameter):
     shortbib: Optional[str] = ""
 
 
+# And a special class for the Spin and Taxonomy lists
+class ParameterList(list):
+    """Subclass of <list> with a custom __str__ for the Spin and Taxonomy parameters."""
+
+    def __init__(self, list_):
+        """Convert the list items to Spin or Taxonomy instances."""
+
+        if "class" in list_[0]:
+            list_ = [Taxonomy(**entry) for entry in list_]
+        else:
+            list_ = [Spin(**entry) for entry in list_]
+
+        return super().__init__(list_)
+
+    def __str__(self) -> str:
+
+        if hasattr(self[0], "class_"):
+            if len(self) == 1:
+                return self[0].class_
+            else:
+                classifications = []
+
+                for entry in self:
+                    shortbib = ", ".join(bib.shortbib for bib in entry.bibref)
+                    classifications.append(f"{entry.class_:<4}{shortbib}")
+
+                return "\n".join(classifications)
+
+        return super().__str__()
+
+
+# ------
+# Validators
+def convert_spin_to_list(spins: Dict) -> List:
+    """Convert the Spin dictionary from the ssoCard into a list.
+    Add the spin index as parameter to the Spin entries.
+
+    Parameters
+    ----------
+    spin : dict
+        The dictionary located at parameters.physical.spin in the ssoCard.
+
+    Returns
+    -------
+    list
+        A list of dictionaries, with one dictionary for each entry in parameters.physical.spin
+        after removing the index layer.
+    """
+
+    spin_dicts = []
+
+    for spin_id, spin_dict in spins.items():
+        spin_dict["id_"] = spin_id
+        spin_dicts.append(spin_dict)
+
+    return spin_dicts
+
+
 # ------
 # Dynamical parameters
 class OrbitalElements(Parameter):
@@ -167,6 +173,11 @@ class Pair(Parameter):
     bibref: List[Bibref] = [Bibref(**{})]
 
 
+class TisserandParameter(Value):
+    method: List[Method] = []
+    bibref: List[Bibref] = []
+
+
 class Yarkovsky(Parameter):
     S: Optional[float] = np.nan
     A2: Value = Value(**{})
@@ -181,6 +192,7 @@ class Yarkovsky(Parameter):
 class DynamicalParameters(Parameter):
     pair: Pair = Pair(**{})
     family: Family = Family(**{})
+    tisserand_parameter: TisserandParameter = TisserandParameter(**{})
     yarkovsky: Yarkovsky = Yarkovsky(**{})
     proper_elements: ProperElements = ProperElements(**{})
     orbital_elements: OrbitalElements = OrbitalElements(**{})
@@ -195,10 +207,6 @@ class Albedo(Value):
     bibref: List[Bibref] = []
     method: List[Method] = []
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", "method", allow_reuse=True, pre=True
-    )(ensure_list)
-
 
 class Color(Value):
     color: Value = Value(**{})
@@ -227,10 +235,6 @@ class Density(Value):
 
     path_unit: str = "unit.physical.density.value"
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", "method", allow_reuse=True, pre=True
-    )(ensure_list)
-
 
 class Diameter(Value):
     method: List[Method] = [Method(**{})]
@@ -267,25 +271,26 @@ class PhaseFunction(Parameter):
 
 
 class Spin(Parameter):
-    period: List[Value] = [Value(**{})]
-    t0: List[float] = [np.nan]
-    Wp: List[float] = [np.nan]
-    lat: List[Value] = [Value(**{})]
-    RA0: List[float] = [np.nan]
-    DEC0: List[float] = [np.nan]
-    long_: List[Value] = pydantic.Field([Value(**{})], alias="long")
-    method: List[List[Method]] = [[Method(**{})]]
-    bibref: List[List[Bibref]] = [[Bibref(**{})]]
+    t0: Optional[float] = np.nan
+    Wp: Optional[float] = np.nan
+    id_: Optional[int] = None
+    lat: Optional[Value] = Value(**{})
+    RA0: Optional[Value] = Value(**{})
+    DEC0: Optional[Value] = Value(**{})
+    long_: Optional[Value] = pydantic.Field(Value(**{}), alias="long")
+    period: Optional[Value] = Value(**{})
+    method: Optional[List[Method]] = [Method(**{})]
+    bibref: Optional[List[Bibref]] = [Bibref(**{})]
 
     path_unit: str = "unit.physical.spin.value"
 
 
 class Taxonomy(Parameter):
-    class_: List[str] = pydantic.Field([""], alias="class")
-    scheme: List[str] = [""]
-    bibref: List[List[Bibref]] = [[Bibref(**{})]]
-    method: List[List[Method]] = [[Method(**{})]]
-    waverange: List[str] = [""]
+    class_: Optional[str] = pydantic.Field("", alias="class")
+    scheme: Optional[str] = ""
+    bibref: Optional[List[Bibref]] = [Bibref(**{})]
+    method: Optional[List[Method]] = [Method(**{})]
+    waverange: Optional[str] = ""
 
     def __str__(self):
         if not self.class_:
@@ -307,23 +312,22 @@ class AbsoluteMagnitude(Value):
 
 class PhysicalParameters(Parameter):
     mass: Mass = Mass(**{})
-    spin: Spin = Spin(**{})
+    spin: ParameterList = ParameterList([{}])
     colors: Colors = Colors(**{})
     albedo: Albedo = Albedo(**{})
     density: Density = Density(**{})
     diameter: Diameter = Diameter(**{})
-    taxonomy: Taxonomy = Taxonomy(**{})
+    taxonomy: ParameterList = ParameterList([{"class": ""}])
     phase_function: PhaseFunction = PhaseFunction(**{})
     thermal_inertia: ThermalInertia = ThermalInertia(**{})
     absolute_magnitude: AbsoluteMagnitude = AbsoluteMagnitude(**{})
 
-    _ensure_list: classmethod = pydantic.validator(
-        "taxonomy", allow_reuse=True, pre=True
-    )(ensure_list)
-
-    _merge_entries: classmethod = pydantic.validator(
+    _convert_spin_to_list: classmethod = pydantic.validator("spin", pre=True)(
+        convert_spin_to_list
+    )
+    _convert_list_to_parameterlist: classmethod = pydantic.validator(
         "spin", "taxonomy", allow_reuse=True, pre=True
-    )(merge_entries)
+    )(lambda list_: ParameterList(list_))
 
 
 # ------
@@ -412,7 +416,7 @@ class Rock(pydantic.BaseModel):
         """Identify a minor body  and retrieve its properties from SsODNet.
 
         Parameters
-        ==========
+        ----------
         id_ : str, int, float
             Identifying asteroid name, designation, or number
         ssocard : dict
@@ -426,17 +430,17 @@ class Rock(pydantic.BaseModel):
             Default is False.
 
         Returns
-        =======
+        -------
         rocks.core.Rock
             An asteroid class instance, with its properties as attributes.
 
         Notes
-        =====
+        -----
         If the asteroid could not be identified or the data contains invalid
         types, the number is None and no further attributes but the name are set.
 
         Example
-        =======
+        -------
         >>> from rocks import Rock
         >>> ceres = Rock('ceres')
         >>> ceres.taxonomy.class_
@@ -619,6 +623,12 @@ class Rock(pydantic.BaseModel):
             for loc in error["loc"]:
                 try:
                     value = value[loc]
+                except KeyError:
+                    # for Spin entries, the location list is messed up
+                    # disabling the entire Spin parameter
+                    if "spin" in error["loc"]:
+                        error["loc"] = ("parameters", "physical", "spin")
+                        break
                 except TypeError:
                     break
 
@@ -658,6 +668,7 @@ class Rock(pydantic.BaseModel):
             "ap": "proper_semi_major_axis",
             "ep": "proper_eccentricity",
             "ip": "proper_inclination",
+            "sinip": "proper_sine_inclination",
         },
         "diamalbedo": ["albedos", "diameters"],
     }
