@@ -1,12 +1,14 @@
 #!/usr/bin/env python
 """Utility functions for rocks."""
 
+import copy
 from functools import reduce
 import json
-import os
 import pickle
+import pickletools
 import re
-import sys
+import string
+import typing
 import warnings
 
 import numpy as np
@@ -49,42 +51,200 @@ def _build_index():
     index_ssodnet.loc[index_ssodnet["Number"] == 0, "Number"] = np.nan
 
     # Reformat the index to a dictionary
-    index_ssodnet = (
-        index_ssodnet.drop(columns="Type")
-        .set_index("SsODNetID")
-        .to_dict(orient="index")
+    index_ssodnet.drop(columns=["Type", "Aliases", "Reduced"])
+    index_ssodnet["Number"] = index_ssodnet["Number"].astype("Int64")
+
+    # ------
+    # Split index into chunks
+
+    # Numbers
+    step = 1e3  # Build chunks of 1k entries
+    number_max = (
+        np.ceil(index_ssodnet.Number.max() / step) * step
+    )  # Find next 10,000 to largest number
+    number_chunks = np.arange(1, number_max, step, dtype=int)
+
+    # Names
+    name_chunks = string.ascii_lowercase  # first character of name
+
+    # Designation
+    designations = set(
+        red
+        for red in index_ssodnet.Reduced
+        if re.match(
+            r"(^([11][8-9][0-9]{2}[a-z]{2}[0-9]{0,3}$)|"
+            r"(^20[0-9]{2}[a-z]{2}[0-9]{0,3}$))",
+            red,
+        )
     )
+    max_year = max([int(year[2:4]) for year in designations if year.startswith("20")])
+    designation_chunks = [
+        "18",
+        "19",
+        *[f"20{year:02}" for year in range(0, max_year)],
+    ]
 
-    INDEX = {"name": {}, "number": {}, "id": {}, "reduced": {}}
+    with progress.Progress() as progress_bar:
+        ntasks = (
+            len(number_chunks) + len(name_chunks) + len(designation_chunks) + 1
+        )  # +1 for the rest
+        task = progress_bar.add_task("Building index", total=ntasks)
 
-    for id_, values in progress.track(
-        index_ssodnet.items(),
-        total=len(index_ssodnet),
-        description="Building Index",
+        # Create number index chunks
+        for number_min in number_chunks:
+
+            index_chunk = {}
+
+            # Asteroids in this chunk
+            asts = index_ssodnet.loc[
+                (number_min <= index_ssodnet.Number)
+                & (index_ssodnet.Number < number_min + step)
+            ]
+
+            for _, ast in asts.iterrows():
+
+                index_chunk[ast.Number] = [ast.Name, ast.SsODNetID]
+
+            with open(rocks.PATH_INDEX / f"{number_min}.pkl", "wb") as file_:
+                index_pickled = pickle.dumps(index_chunk, protocol=4)
+                index_pickled_opt = pickletools.optimize(index_pickled)
+                file_.write(index_pickled_opt)
+            progress_bar.update(task, advance=1)
+
+        # -----
+        # Reduced
+
+        # Names
+        names = set(
+            red for red in index_ssodnet.Reduced if re.match(r"^[a-z\'-]*$", red)
+        )
+        names.add(r"g!kun||'homdima")  # everyone's favourite shell injection
+
+        # Create name index chunks
+        for char in name_chunks:
+
+            index_chunk = {}
+
+            names_subset = set(name for name in names if name.startswith(char))
+
+            # Asteroids in this chunk
+            asts = index_ssodnet.loc[index_ssodnet.Reduced.isin(names_subset)]
+
+            for _, ast in asts.iterrows():
+
+                if pd.isna(ast.Number):
+                    index_chunk[ast.Reduced] = [ast.Name, ast.SsODNetID]
+                else:
+                    index_chunk[ast.Reduced] = [ast.Name, ast.Number, ast.SsODNetID]
+
+            with open(rocks.PATH_INDEX / f"{char}.pkl", "wb") as file_:
+                index_pickled = pickle.dumps(index_chunk, protocol=4)
+                index_pickled_opt = pickletools.optimize(index_pickled)
+                file_.write(index_pickled_opt)
+
+            progress_bar.update(task, advance=1)
+        # Designations
+
+        # Create designation index chunks
+        for year in designation_chunks:
+
+            index_chunk = {}
+
+            designations_subset = set(
+                desi for desi in designations if desi.startswith(year)
+            )
+
+            # Asteroids in this chunk
+            asts = index_ssodnet.loc[index_ssodnet.Reduced.isin(designations_subset)]
+
+            for _, ast in asts.iterrows():
+
+                if pd.isna(ast.Number):
+                    index_chunk[ast.Reduced] = [ast.Name, ast.SsODNetID]
+                else:
+                    index_chunk[ast.Reduced] = [ast.Name, ast.Number, ast.SsODNetID]
+
+            with open(rocks.PATH_INDEX / f"{year}.pkl", "wb") as file_:
+                index_pickled = pickle.dumps(index_chunk, protocol=4)
+                index_pickled_opt = pickletools.optimize(index_pickled)
+                file_.write(index_pickled_opt)
+
+            progress_bar.update(task, advance=1)
+        # And the rest
+        rest = set(
+            red
+            for red in index_ssodnet.Reduced
+            if red not in names and red not in designations
+        )
+
+        index_chunk = {}
+
+        # Asteroids in this chunk
+        asts = index_ssodnet.loc[index_ssodnet.Reduced.isin(rest)]
+
+        for _, ast in asts.iterrows():
+
+            if pd.isna(ast.Number):
+                index_chunk[ast.Reduced] = [ast.Name, ast.SsODNetID]
+            else:
+                index_chunk[ast.Reduced] = [ast.Name, ast.Number, ast.SsODNetID]
+
+        with open(rocks.PATH_INDEX / f"PLT.pkl", "wb") as file_:
+            index_pickled = pickle.dumps(index_chunk, protocol=4)
+            index_pickled_opt = pickletools.optimize(index_pickled)
+            file_.write(index_pickled_opt)
+
+        progress_bar.update(task, advance=1)
+
+
+def get_index_file(id_: typing.Union[int, str]) -> dict:
+    """Get absolute path to the index chunk where id_ should be located it.
+
+    Parameters
+    ----------
+    id_ : str, int
+        The asteroid identifier.
+
+    Returns
+    -------
+    str
+        The absolute path to the index chunk.
+    """
+
+    # Is it numeric?
+    if isinstance(id_, int):
+        index_range = np.arange(1, int(1e6), int(1e3))
+        try:
+            index_number = index_range[index_range <= id_][-1]
+        # The passed id_ is larger than 1e6
+        except IndexError:
+            index_number = 1
+        path = rocks.PATH_INDEX / f"{index_number}.pkl"
+
+    # Is it a name?
+    elif re.match(r"^[a-z\'-]*$", id_) or id_ == r"g!kun||'homdima":
+        path = rocks.PATH_INDEX / f"{id_[0]}.pkl"
+
+    # Is it a designation?
+    elif re.match(
+        r"(^([11][8-9][0-9]{2}[a-z]{2}[0-9]{0,3}$)|"
+        r"(^20[0-9]{2}[a-z]{2}[0-9]{0,3}$))",
+        id_,
     ):
+        if id_.startswith("20"):
+            year = f"20{id_[2:4]}"
+        else:
+            year = id_[:2]
+        path = rocks.PATH_INDEX / f"{year}.pkl"
 
-        name = values["Name"]
-        number = values["Number"]
-        reduced = reduce_id(id_)
+    # Should be in this one then
+    else:
+        path = rocks.PATH_INDEX / f"PLT.pkl"
 
-        if not np.isnan(number):
-            number = int(number)
-
-        INDEX["name"][name] = (number, id_)
-        INDEX["id"][id_] = (name, number)
-        INDEX["reduced"][reduced] = (name, number, id_)
-
-        if not np.isnan(number):
-            INDEX["number"][number] = (name, id_)
-
-    with open(rocks.PATH_INDEX, "wb") as file_:
-        pickle.dump(INDEX, file_, protocol=4)
-
-
-def load_index():
-    """Load local index of asteroid numbers, names, SsODNet IDs."""
-    with open(rocks.PATH_INDEX, "rb") as ind:
-        return pickle.load(ind)
+    if not path in rocks.INDEX:
+        with open(path, "rb") as ind:
+            rocks.INDEX[path] = pickle.load(ind)
+    return rocks.INDEX[path]
 
 
 # ------
@@ -98,21 +258,21 @@ def rgetattr(obj, attr):
     return reduce(_getattr, [obj] + attr.split("."))
 
 
-def get_unit(path_unit):
+def get_unit(path_unit: str) -> str:
     """Get unit from units JSON file.
 
     Parameters
-    ==========
+    ----------
     path_unit : str
         Path to the parameter in the JSON tree, starting at unit and
         separating the levels with periods.
 
     Returns
-    =======
+    -------
     str
         The unit of the requested parameter.
     """
-    if not os.path.isfile(rocks.PATH_META["units"]):
+    if not rocks.PATH_META["units"].is_file():
         retrieve_json_from_ssodnet("units")
 
     with open(rocks.PATH_META["units"], "r") as units:
@@ -130,14 +290,14 @@ def weighted_average(catalogue, parameter):
     """Computes weighted average of observable.
 
     Parameters
-    ==========
+    ----------
     observable : np.ndarray
         Float values of observable
     error : np.ndarray
         Corresponding errors of observable.
 
     Returns
-    =======
+    -------
     float
         The weighted average.
 
@@ -145,13 +305,14 @@ def weighted_average(catalogue, parameter):
         The standard error of the weighted average.
     """
 
+    values = catalogue[parameter]
+
     if parameter in ["albedo", "diameter"]:
         preferred = catalogue[f"preferred_{parameter}"]
+        errors = catalogue[f"err_{parameter}_up"]
     else:
         preferred = catalogue["preferred"]
-
-    values = catalogue[parameter]
-    errors = catalogue[f"err_{parameter}"]
+        errors = catalogue[f"err_{parameter}"]
 
     observable = np.array(values)[preferred]
     error = np.array(errors)[preferred]
@@ -249,57 +410,54 @@ def cache_inventory():
     """
 
     # Get all JSONs in cache
-    cached_jsons = set(
-        file_ for file_ in os.listdir(rocks.PATH_CACHE) if file_.endswith(".json")
-    )
+    cached_jsons = set(file_ for file_ in rocks.PATH_CACHE.glob("*.json"))
 
     cached_cards = []
     cached_catalogues = []
 
     for file_ in cached_jsons:
 
-        if file_ in [os.path.basename(f) for f in rocks.PATH_META.values()]:
+        # Is it metadata?
+        if file_ in rocks.PATH_META.values():
             continue
 
-        # Datacloud catalogue?
+        # Datacloud catalogue or ssoCard?
         if any(
             [
-                cat["ssodnet_name"] in file_
+                cat["ssodnet_name"] in str(file_)
                 for cat in rocks.datacloud.CATALOGUES.values()
             ]
         ):
+            *ssodnet_id, catalogue = file_.stem.split("_")
+            ssodnet_id = "_".join(ssodnet_id)
+        else:
+            ssodnet_id = file_.stem
+            catalogue = ""
 
-            ssodnet_id = "_".join(file_.split("_")[:-1])
-            catalogue = os.path.splitext(file_)[0].split("_")[-1]
+        # Is it valid?
+        with open(file_, "r") as ssocard:
 
-            cached_catalogues.append((ssodnet_id, catalogue))
-            continue
-
-        # Likely an ssoCard, check the version
-        ssodnet_id = os.path.splitext(file_)[0]
-
-        with open(os.path.join(rocks.PATH_CACHE, file_), "r") as ssocard:
             card = json.load(ssocard)
 
-            try:
-                if card[ssodnet_id] is None:
-                    # Faulty card, remove it
-                    os.remove(os.path.join(rocks.PATH_CACHE, file_))
-                else:
-                    cached_cards.append(
-                        (
-                            ssodnet_id,
-                            card[ssodnet_id]["ssocard"]["version"],
-                        )
-                    )
-            # This skips files which do not belong to rocks
-            except KeyError:
+            if not card:
+                # Empty card or catalogue, remove it
+                file_.unlink()
                 continue
 
+            if not catalogue:
+                if card[ssodnet_id] is None:
+                    # Faulty card, remove it
+                    file_.unlink()
+                    continue
+
+        # Append to inventory
+        if catalogue:
+            cached_catalogues.append((ssodnet_id, catalogue))
+        else:
+            cached_cards.append(ssodnet_id)
+
     # Get cached metadata files
-    cached_meta = [
-        os.path.basename(f) for f in rocks.PATH_META.values() if os.path.isfile(f)
-    ]
+    cached_meta = [f.stem for f in rocks.PATH_META.values() if f.is_file()]
     return cached_cards, cached_catalogues, cached_meta
 
 
@@ -308,42 +466,16 @@ def clear_cache():
     cards, catalogues, _ = cache_inventory()
 
     for card in cards:
-        PATH_CARD = os.path.join(rocks.PATH_CACHE, f"{card[0]}.json")
-        os.remove(PATH_CARD)
+        PATH_CARD = rocks.PATH_CACHE / f"{card}.json"
+        PATH_CARD.unlink()
 
     for catalogue in catalogues:
-        PATH_CATALOGUE = os.path.join(rocks.PATH_CACHE, f"{'_'.join(catalogue)}.json")
-        os.remove(PATH_CATALOGUE)
+        PATH_CATALOGUE = rocks.PATH_CACHE / f"{'_'.join(catalogue)}.json"
+        PATH_CATALOGUE.unlink()
 
     for file_ in rocks.PATH_META.values():
-        if os.path.isfile(file_):
-            os.remove(file_)
-
-
-def get_current_version():
-    """Get the current version of ssoCards.
-
-    Returns
-    -------
-    str
-        The current version of ssoCards.
-
-    Notes
-    -----
-    There will soon be a stub card online to check this. For now, we just check
-    the version of Ceres.
-    """
-
-    URL = "https://ssp.imcce.fr/webservices/ssodnet/api/ssocard/Ceres"
-    response = requests.get(URL)
-
-    if response.ok:
-        card = response.json()
-    else:
-        warnings.warn("Retrieving the current ssoCard version failed.")
-        sys.exit()
-
-    return card["Ceres"]["ssocard"]["version"]
+        if file_.is_file():
+            file_.unlink()
 
 
 def update_datacloud_catalogues(cached_catalogues):
@@ -361,7 +493,7 @@ def confirm_identity(ids):
     ssoCard and remove the former one if the ID has changed.
 
     Parameters
-    ==========
+    ----------
     ids : list
         The list of SsODNet IDs to confirm.
     """
@@ -374,9 +506,7 @@ def confirm_identity(ids):
     elif len(ids) == 1:
         _, _, current_ids = rocks.identify(ids, return_id=True, local=False)
         current_ids = [current_ids]
-
     else:
-
         _, _, current_ids = zip(
             *rocks.identify(ids, return_id=True, local=False, progress=True)
         )
@@ -393,7 +523,7 @@ def confirm_identity(ids):
 
         # Get new card and remove the old one
         rocks.ssodnet.get_ssocard(current_id, local=False)
-        os.remove(os.path.join(rocks.PATH_CACHE, f"{old_id}.json"))
+        (rocks.PATH_CACHE / f"{old_id}.json").unlink()
 
         # This is now up-to-date
         updated.append(old_id)
@@ -413,7 +543,7 @@ def retrieve_rocks_version():
         version = ""
     finally:
         if response.status_code == 200:
-            version = re.findall(r"\d\.\d\.?\d?", response.text)[0]
+            version = re.findall(r"\d+\.\d+[\.\d]*", response.text)[0]
         else:
             version = ""
 
@@ -439,19 +569,41 @@ def find_candidates(id_):
     """
 
     # Get list of named asteroids
-    index = load_index()
+    index = {}
+
+    for char in string.ascii_lowercase:
+        idx = get_index_file(char)
+        index = {**index, **idx}
 
     # Use Levenshtein distance to identify potential matches
     candidates = []
     max_distance = 1  # found by trial and error
-    id_ = id_.capitalize()  # remove one possible dof
+    id_ = reduce_id(id_)
 
-    for name in index["name"].keys():
+    for name in index.keys():
         distance = lev.distance(id_, name)
 
         if distance <= max_distance:
-            candidates.append((name, index["name"][name][0]))
+            candidates.append(index[name][:-1])
 
     # Sort by number
     candidates = sorted(candidates, key=lambda x: x[1])
     return candidates
+
+
+def list_candidate_ssos(id_):
+    # The query failed. Propose some names if the id_ looks like a name,
+    # designations give too many false positives
+    if not re.match(r"^[A-Za-z ]*$", id_):
+        return
+
+    candidates = rocks.utils.find_candidates(id_)
+
+    if candidates:
+        rich.print(
+            f"\nCould {'this' if len(candidates) == 1 else 'these'} be the "
+            f"{'rock' if len(candidates) == 1 else 'rocks'} you're looking for?"
+        )
+
+        for name, number in candidates:
+            rich.print(f"{f'({number})':>8} {name}")
